/*
 * M. Mueller, 05/05/93
 *
 * Switch user id to root and execute command. If command is
 * not given $SHELL or sh is started. The calling user must be in the users[]
 * vector below to have permission.
 *
 * Syntax:
 *	srt [command ...]
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <pwd.h>
#include <grp.h>

/* allowed users */
char *users[] = {
	"root",
	"kurs",
};
#define NUSERS (sizeof(users) / sizeof(char *))

static void getid(char *name, int *pUid, int *pGid)
{
	struct passwd *p;

	if ( !(p=getpwnam(name)) ) {
		fprintf(stderr, "user %s not found\n", name );
		exit(1);
	}

	*pUid = p->pw_uid;
	*pGid = p->pw_gid;
}

static int allowed()
{
	int n;
	int uid, gid;
	int myid = getuid();

	for (n = 0; n < NUSERS; ++n) {
		getid(users[n], &uid, &gid);
		if (uid == myid)
			return 1;
	}

	return 0;
}

main(int argc, char **argv)
{
	char *sh, *p, *p2;
	int uid, gid, h;
	struct group *pGrp;

	if (!allowed())
		exit(0);

	for( p=p2=argv[0]; *p2; ++p2 )
		if ( *p2 == '/' )
			p = p2 + 1;

	getid("root", &uid, &gid );

	setgid(gid);
	setuid(uid);


	if ( argc < 2 ) {
		if ( !(sh=getenv("SHELL")) )
			sh="sh";

		execlp( sh, sh, NULL );
		fprintf(stderr, "cannot execute %s\n", sh);
	}
	else
	{
		++argv;
		execvp(argv[0], argv);
		fprintf(stderr, "cannot execute %s\n", argv[0]);

	}
}
