/*
 * usage: looptst2 [num [-n] ]
 *
 * Do num writes and reads in a loop to test the loopback device. In the end 
 * use a high priority message to get the device status. With -n the read is
 * suppressed and if num is high enough, the output becomes blocked. This is
 * detected by a poll before the read and the loop is aborted.
 */

#include <sys/types.h>
#include <stropts.h>
#include <sys/conf.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <poll.h>

#include "loop.h"

void
printstat(int fd)
{
	struct loop_statistics stat;
	struct strbuf ctlbuf, databuf;
	struct loop_ctl loop_ctl;
	int flag;

	loop_ctl.cmd = LOOP_C_STAT;
	ctlbuf.buf = (char *)&loop_ctl;
	ctlbuf.len = sizeof(loop_ctl);

	databuf.buf = (char *)&stat;
	databuf.len = sizeof(stat);

	printf("statistics:\n");

	if (putmsg(fd, &ctlbuf, &databuf, RS_HIPRI) < 0) {
		perror("putmsg");
		return;
	}

	ctlbuf.maxlen = sizeof(loop_ctl);
	databuf.maxlen = sizeof(stat);
	flag = RS_HIPRI;
	if (getmsg(fd, &ctlbuf, &databuf, &flag) < 0) {
		perror("getmsg");
		return;
	}
	if (loop_ctl.error) {
		char *p = strerror(loop_ctl.error);
		printf("LOOP_C_STAT error %d: %s\n", loop_ctl.error,
			 p ? p : "???");
			return;
	}

	printf("   num_msg %d, num_allocb %d, num_nomem %d\n",
		stat.num_msg, stat.num_allocb, stat.num_nomem);
	printf("   q_count %d, q_mblkcnt %d\n",
		stat.q_count, stat.q_mblkcnt);
}

main(int argc, char **argv)
{
	int fd, n, l, num;
	char s[80];
	int noread = 0;
	struct pollfd fds[1];

	if (argc < 2)
		num = 10;
	else
		num = atoi(argv[1]);
	if (argc >= 3)
		noread = 1;
	printf("num %d, noread %d\n", num, noread);

	fd = open(LOOPDEV, O_RDWR);
	if (fd < 0) {
		perror("open");
		exit(1);
	}

	/* close timeout in ms if write q is not empty, default = 15 sec */
	n = 100;
	if (ioctl(fd, I_SETCLTIME, &n) < 0) {
		perror("I_SETCLTIME");
		exit(1);
	}

	for (n = 0; n < num; ++n) {
		sprintf(s, "message %d", n);
		l = strlen(s);

		fds[0].fd = fd;
		fds[0].events = POLLOUT;
		if (!poll(fds, 1, 100)) {
			printf("output is blocked\n");
			break;
		}

		if (write(fd, s, l) != l) {
			perror("write");
			exit(1);
		}
		printf("written: %s", s);

		if (noread )
			printf("\n");
		else {
			l = read(fd, s, sizeof(s) - 1);
			if (l < 0) {
				perror("read");
				exit(1);
			}
			s[l] = 0;
			printf(",\tread: %s\n", s);
		}
	}

	printstat(fd);

	return 0;
}
