/*
 */

#include <sys/types.h>
#include <stropts.h>
#include <sys/conf.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mkdev.h>

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "mux.h"
#include "loop.h"

void Error(char *str, ...);
void Usage(void);
void read_messages(int cnt);
void write_messages(int cnt, int down, int up);

int min;
int fd;

int
main(int argc, char **argv)
{
	int n;
	int cnt, up, down;
	struct stat sbuf;
	char buf[100], *p;
	char c;

	fd = open(MUXDEV, O_RDWR);
	if (fd < 0)
		Error("cannot open %s", MUXDEV);

	if (fstat(fd, &sbuf) < 0)
		Error("cannot stat %s", MUXDEV);
	min = minor(sbuf.st_rdev);
	printf("major %d, minor %d\n", major(sbuf.st_rdev), min);

	n = 100;
	if (ioctl(fd, I_SETCLTIME, &n) < 0)
		Error("ioctl(I_SETCLTIME)");
		
	for (;;) {
		printf("(h = help) > ");
		if (!fgets(buf, sizeof(buf), stdin))
			break;
		
		n = sscanf(buf, " %c%d%d%d", &c, &cnt, &down, &up);
		if (n < 1)
			c = ' ';

		switch (c) {
		    case 'q':
			return 0;
		    case 'r':
			if (n < 2) {
				Usage();
				break;
			}
			read_messages(cnt);
			break;
		    case 'w':
			if (n < 4) {
				Usage();
				break;
			}
			write_messages(cnt, down, up);
			break;
		
		    case 'h':
		    default:
			Usage();
			break;
		}
	}

	return 0;
}

void read_messages(int cnt)
{
	struct strbuf ctlbuf, databuf;
	struct mux_ctl ctl;
	char s[80];
	int flag;
	int n;

	for (n = 0; n < cnt; ++n) {
		ctlbuf.maxlen = sizeof(ctl);
		ctlbuf.buf = (char *)&ctl;

		databuf.maxlen = sizeof(s) - 1;
		databuf.buf = (char *)&s;

		flag = 0;

		if (getmsg(fd, &ctlbuf, &databuf, &flag) < 0)
			Error("getmsg");
		s[databuf.len] = 0;
		printf("read: %s", s);
		if (ctl.error)
			printf(", ctl.error %d\n", ctl.error);
		printf("\n");
	}

}

void write_messages(int cnt, int down, int up)
{
	static int msgcnt = 0;

	struct strbuf ctlbuf, databuf;
	struct mux_ctl ctl;
	char s[80];
	int n;

	for (n = 0; n < cnt; ++n) {
		ctl.cmd = MUX_C_ROUTE;
		ctl.down_idx = down;
		ctl.up_idx = up;
		ctlbuf.buf = (char *)&ctl;
		ctlbuf.len = sizeof(ctl);

		sprintf(s, "Message %d from minor %d, down_idx %d up_idx %d ",
			++msgcnt, min, down, up);

		databuf.buf = (char *)&s;
		databuf.len = strlen(s);

		if (putmsg(fd, &ctlbuf, &databuf, 0) < 0)
			Error("putmsg");
		printf("written: %s\n", s);
	}
}

void
Error(char *str, ...)
{
        va_list ap;
	int e;

	e = errno;

        va_start(ap, str);
        vfprintf(stderr, str, ap);
	va_end(ap);
        fprintf( stderr, "\n" );

	if (e)
		fprintf(stderr, "system error %d: %s\n", e, strerror(e));

        exit(1);
}

void Usage(void)
{
	printf("   write: w <num-writes> <l_index> <upper-minor>\n");
	printf("   read:  r <num-reads>\n");
	printf("   quit:  q\n");
	printf("   help:  h\n");
}
