/*
 * Link /dev/loop under /dev/sspymux to test sspymux.
 *
 * usage: looptst3
 *
 * Do writes in a loop until output q is blocked using poll. Get q status,
 * read all pending messages and do one more write / read pair to see if
 * output is backenabled.
 */

#include <sys/types.h>
#include <stropts.h>
#include <sys/conf.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <poll.h>

#include "loop.h"
#include "sspy.h"

void
printstat(int fd)
{
	struct loop_statistics stat;
	struct strbuf ctlbuf, databuf;
	struct loop_ctl loop_ctl;
	int flag;

	loop_ctl.cmd = LOOP_C_STAT;
	ctlbuf.buf = (char *)&loop_ctl;
	ctlbuf.len = sizeof(loop_ctl);

	databuf.buf = (char *)&stat;
	databuf.len = sizeof(stat);

	printf("statistics:\n");

	if (putmsg(fd, &ctlbuf, &databuf, RS_HIPRI) < 0) {
		perror("putmsg");
		return;
	}

	ctlbuf.maxlen = sizeof(loop_ctl);
	databuf.maxlen = sizeof(stat) * 2;
	databuf.len = sizeof(stat) * 2;
	flag = RS_HIPRI;
	if (getmsg(fd, &ctlbuf, &databuf, &flag) != 0) {
		perror("getmsg");
		return;
	}
	if (loop_ctl.error) {
		char *p = strerror(loop_ctl.error);
		printf("LOOP_C_STAT error %d: %s\n", loop_ctl.error,
			p ? p : "???");
			return;
	}

	printf("   num_msg %d, num_allocb %d, num_nomem %d\n",
		stat.num_msg, stat.num_allocb, stat.num_nomem);
	printf("   q_count %d, q_mblkcnt %d\n",
		stat.q_count, stat.q_mblkcnt);
}

main(int argc, char **argv)
{
	int fd, n, l;
	char s[80], *p;
	struct pollfd fds[1];

	fd = open(SSPYMUXDEV, O_RDWR);
	if (fd < 0) {
		perror("open");
		exit(1);
	}

	/* close timeout in ms if write q is not empty, default = 15 sec */
	n = 100;
	if (ioctl(fd, I_SETCLTIME, &n) < 0) {
		perror("I_SETCLTIME");
		exit(1);
	}

	for (n = 0; ; ++n) {
		sprintf(s, "this is message %d !!!\n", n);
		l = strlen(s);

		fds[0].fd = fd;
		fds[0].events = POLLOUT;
		if (!poll(fds, 1, 100)) {
			printf("output is blocked\n");
			break;
		}

		if (write(fd, s, l) != l) {
			perror("write");
			exit(1);
		}
		printf("written: %s", s);
	}

	printstat(fd);

	printf("reading pending messages\n");
	for (;;) {
		fds[0].fd = fd;
		fds[0].events = POLLIN;
		if (!poll(fds, 1, 100)) {
			printf("\ninput is blocking\n");
			break;
		}

		l = read(fd, s, sizeof(s) - 1);
		if (l < 0) {
			perror("read");
			exit(1);
		}
		s[l] = 0;
		printf("%s", s);
	}

	p = "this is a last message";
	l = strlen(p);
	if (write(fd, p, l) != l) {
		perror("write");
		exit(1);
	}
	printf("written: %s\n", p);

	l = read(fd, s, sizeof(s) - 1);
	if (l < 0) {
		perror("read");
		exit(1);
	}
	s[l] = 0;
	printf("read: %s\n", s);
	printstat(fd);

	return 0;
}
